{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
    "###### Elevator Simulation\n",
    "## CS166: Modeling, Simulation & Decision Making"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 58,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
    "import math\n",
    "import numpy as np\n",
    "import random\n",
    "import matplotlib.pyplot as plt\n",
    "\n",
    "from scipy.stats import ttest_ind\n",
    "from matplotlib import ticker\n",
    "from copy import deepcopy\n",
    "\n",
    "%matplotlib inline\n",
    "plt.rcParams['figure.figsize'] = (18, 4)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 59,
   "metadata": {
    "collapsed": false,
    "scrolled": true
   },
   "outputs": [
   ],
   "source": [
    "# # Simulation parameters\n",
    "# passenger_count = int(input(\"How many passengers will use the elevator?\"))\n",
    "# max_capacity = int(input(\"How many people can the elevator carry?\"))\n",
    "# n_floors = int(input(\"How many floors are there in the building?\"))\n",
    "# max_time = range(1800) # Time steps\n",
    "\n",
    "passenger_count = 100\n",
    "max_capacity = 5\n",
    "n_floors = 20\n",
    "max_time = range(1800) # Time steps"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "collapsed": false
   },
   "source": [
    "Definition of the elevator class"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 60,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
    "class Elevator:\n",
    "\n",
    "    def __init__(self, max_floors = n_floors, default_floor = 1, capacity = max_capacity, logs = False):\n",
    "        '''\n",
    "        Initializes an elevator object. \n",
    "        Inputs:\n",
    "        \n",
    "         * max_floors (int) The number of floors in the building. \n",
    "         \n",
    "         * default_floor (int) The floor the elevator returns to if\n",
    "           there are no calls or requests.\n",
    "           \n",
    "         * capacity (int) The maximum number of people the elevator\n",
    "           can carry at the same time.\n",
    "           \n",
    "         * logs (bool) Whether to print several debugging messages\n",
    "           when running the simulation.\n",
    "           \n",
    "        '''\n",
    "\n",
    "        self.max_floors = max_floors\n",
    "        self.default_floor = default_floor\n",
    "        self.current_floor = self.default_floor\n",
    "        self.user_count = 0 # How many users are in the elevator.\n",
    "        self.capacity = capacity #OR max_capacity = int(input(\"How many people can the elevator carry?\"))\n",
    "        self.call_pool = {} \n",
    "        self.calls = {} # which user pressed the button and attempting to go in which direction\n",
    "        self.requests = {} # which buttons were pressed inside the elevator by whom\n",
    "        self.direction = +1 # +1 if the elevator is going upward, -1 if going downward\n",
    "\n",
    "##Nico's methods    \n",
    "    \n",
    "    def add_call(self):\n",
    "        '''\n",
    "        At each time step, there is a chance that will method will be called from\n",
    "        the building method timestep(), adding a random user to the list of calls.\n",
    "        '''\n",
    "        # Choose a random user from pool. Assign the popped user to dictionary of elevator calls.\n",
    "        calling_user = random.choice(list(self.call_pool.keys()))\n",
    "        self.calls[calling_user] = self.call_pool.pop(calling_user) #remove one call from the call_pool, append it to the elevator calls\n",
    "        return calling_user\n",
    "\n",
    "    def stop(self):\n",
    "        '''\n",
    "        At each time step, this method is called to evaluated whether the elevator\n",
    "        should stop at the current floor. Returns a boolean that answers the \n",
    "        question: Should the elevator stop?\n",
    "        '''\n",
    "        calls_copy = list(self.calls.values())\n",
    "        calls_copy = [abs(call) for call in calls_copy]\n",
    "        # Checks whether current floor is the desired destination of any users inside.\n",
    "        if (self.current_floor in self.requests.values()):\n",
    "            return True\n",
    "        \n",
    "        # Checks whether current floor and direction matches the call of any users outside, if capacity has not been reached.\n",
    "        elif (self.direction*self.current_floor in self.calls.values()) and (self.user_count < self.capacity):\n",
    "            return True\n",
    "        \n",
    "        elif (self.current_floor in calls_copy) and (self.user_count == 0):\n",
    "            return True\n",
    "        else:\n",
    "            return False # Move elevator\n",
    "    \n",
    "    def swap(self, users, logs = False):\n",
    "        '''\n",
    "        At each time step, if the elevator stops, this method allows users to\n",
    "        leave or enter the elevator.\n",
    "        ''' \n",
    "        \n",
    "        #we want to pop things from dictionary while we iterate\n",
    "        calls_copy = deepcopy(self.calls)\n",
    "        requests_copy = deepcopy(self.requests)\n",
    "        \n",
    "        # Allows people to leave, iterating over passengers and floors in the dictionary of requests.\n",
    "        for passenger, floor in requests_copy.items():\n",
    "            if floor == self.current_floor:\n",
    "                self.requests.pop(passenger) # Return dropped passenger and remove it from queue of requests_over_time\n",
    "                dropped_passenger = passenger\n",
    "                # Question: Can't we just pop and use the object passenger to set .in_elevator to False?\n",
    "                self.user_count -=1\n",
    "                if logs: print('Passenger %d was dropped at floor %d.' %(passenger, self.current_floor))\n",
    "                users[dropped_passenger].in_elevator = False # dropped passenger is now outside the elevator\n",
    "        \n",
    "        # Allows people to enter, iterating over passengers and floors in the dictionary of calls.\n",
    "        for passenger, floor in calls_copy.items():\n",
    "            if self.user_count == self.capacity: # If the elevator is full, break.\n",
    "                break\n",
    "            \n",
    "            elif self.user_count == 0:\n",
    "                if abs(floor) == self.current_floor:\n",
    "                    self.calls.pop(passenger) # Return added passenger and remove it from queue of calls\n",
    "                    added_passenger = passenger\n",
    "                    self.user_count +=1\n",
    "                    if logs: print('Passenger %d entered the elevator at floor %d.' %(passenger, self.current_floor))\n",
    "                    users[added_passenger].in_elevator = True # Added passenger is now inside the elevator\n",
    "                    self.requests[passenger] = users[passenger].destination # Add their destination to requests\n",
    "                    if logs: print(users[passenger].destination,'DESTINATION OF PASSENGER',passenger)   \n",
    "            \n",
    "            elif floor == self.direction*self.current_floor: # If floor and direction match call\n",
    "                self.calls.pop(passenger) # Return added passenger and remove it from queue of calls\n",
    "                # Same question from loop above applies.\n",
    "                added_passenger = passenger\n",
    "                self.user_count +=1\n",
    "                if logs: print('Passenger %d entered the elevator at floor %d.' %(passenger, self.current_floor))\n",
    "                users[added_passenger].in_elevator = True # Added passenger is now inside the elevator\n",
    "                self.requests[passenger] = users[passenger].destination # Add their destination to requests\n",
    "                if logs: print(users[passenger].destination,'DESTINATION OF PASSENGER',passenger)\n",
    "\n",
    "    def path(self):\n",
    "        '''\n",
    "        This strategy looks at each time step where it has to move next. \n",
    "        It takes into account the calls and requests, prioritizing requests over calls. \n",
    "        If there are no requests or calls it goes to the default (ground) floor.\n",
    "        If there are no requests, but calls are present, it prioritizes the calls that are going to **sweep** the most floors while moving.\n",
    "        \n",
    "        '''\n",
    "\n",
    "        # Create a list of all the called floors\n",
    "        current_calls = list(self.calls.values())\n",
    "\n",
    "        if self.current_floor >= self.max_floors: # >= for debugging\n",
    "            self.direction = -1 # Direction is either -1 or 1\n",
    "\n",
    "        # If there are no requests from inside the elevator, pursue the strategy\n",
    "        elif len(self.requests) == 0:\n",
    "\n",
    "            # If there are no calls, go to default floor\n",
    "            if len(current_calls) == 0:\n",
    "                if self.current_floor-self.default_floor > 0:\n",
    "                    self.direction = -1\n",
    "                # If already on default floor, stay\n",
    "                else:\n",
    "                    self.direction = 0\n",
    "\n",
    "            # If there are calls outside the elevator\n",
    "            else:\n",
    "                #Get the min of all calls. Negative numbers are calls going down, so it will prioritize high floors going down\n",
    "                negmax = min(current_calls)\n",
    "                #Create a list with only the calls going up\n",
    "\n",
    "                poscalls = [i for i in current_calls if i > 1]\n",
    "                if len(poscalls)>0:\n",
    "                    #Min of the list of calls going up, that way you end up with the lowest floor calling to go up\n",
    "                    posmin = min(poscalls)\n",
    "                #If there's only 1 call and no requests, go towards the called floor\n",
    "                if len(current_calls) == 1:    \n",
    "               \n",
    "                    if current_calls[0]>0:\n",
    "                        direction = self.current_floor - current_calls[0]\n",
    "                        if direction >0:\n",
    "                            self.direction = -1\n",
    "                        else:\n",
    "                            self.direction = +1\n",
    "                    else:\n",
    "                        direction = self.current_floor + current_calls[0]\n",
    "                        if direction >0:\n",
    "                            self.direction = -1\n",
    "                        else:\n",
    "                            self.direction = +1       \n",
    "                           \n",
    "                elif 1 in current_calls:\n",
    "                    #If multiple calls, prioritize the top or ground floor. If both are being called, then go go the one closest to current floor.\n",
    "                    if self.max_floors in current_calls:\n",
    "                        if current_floor<=(0.5*self.max_floors):\n",
    "                           \n",
    "                            self.direction = -1\n",
    "                        else:\n",
    "                            self.direction = +1\n",
    "                elif self.max_floors in current_calls:\n",
    "                    self.direction = +1\n",
    "                #If there are no calls going up just go towards the highest floor being called\n",
    "                elif len(poscalls) == 0:\n",
    "                    direction = self.current_floor + negmax\n",
    "                    if direction > 0:\n",
    "                        self.direction = -1\n",
    "                    else:\n",
    "                        self.direction = +1\n",
    "                #If there are only calls going up, go towards the lowest up call.    \n",
    "                elif posmin == negmax:\n",
    "                    direction = self.current_floor - negmax\n",
    "                    if direction > 0:\n",
    "                        self.direction = -1\n",
    "                    else:\n",
    "                        self.direction = +1   \n",
    "                #If both calls going up and down are present, choose the one that is closest to the ground floor or top floor, respectively.\n",
    "                elif (posmin-1) < (self.max_floors + negmax):\n",
    "                    if self.current_floor < posmin:\n",
    "                        self.direction = +1\n",
    "                    else:\n",
    "                        self.direction = -1\n",
    "                elif (posmin-1) > (self.max_floors + negmax):\n",
    "\n",
    "                    if self.current_floor>(negmax*-1):\n",
    "                        self.direction = -1\n",
    "                    else:\n",
    "                        self.direction = +1               \n",
    "                else:\n",
    "                    #If the distance between the calls going up and down to its 'roof floors' are the same, \n",
    "                    #prioritize the one closest to current floor\n",
    "                    pos = abs(self.current_floor - posmin)\n",
    "                    neg = abs(self.current_floor - negmax)\n",
    "                    if pos<neg:\n",
    "                        if (self.current_floor - posmin) >0:\n",
    "                            self.direction = -1\n",
    "                        else:\n",
    "                            self.direction = +1\n",
    "                    else:\n",
    "                        if (self.current_floor + negmax) >0:\n",
    "                            self.direction = -1\n",
    "                        else:\n",
    "                            self.direction = +1\n",
    "\n",
    "        else:\n",
    "            #If there are requests, go towards any requested floor, as they are all in the same direction.\n",
    "            request1 = list(self.requests.values())\n",
    "            direction = self.current_floor - request1[0]\n",
    "            if direction < 0:\n",
    "                self.direction = +1\n",
    "            else:\n",
    "\n",
    "                self.direction = -1\n",
    "\n",
    "\n",
    "    def move(self, strategy = 1, logs = False):\n",
    "        '''\n",
    "        At each time step, move the elevator if it has not stopped.\n",
    "        '''\n",
    "                \n",
    "        if self.direction == +1:\n",
    "            self.current_floor += 1\n",
    "        elif self.direction == -1:\n",
    "            self.current_floor -= 1\n",
    "            \n",
    "        if self.direction == 1:\n",
    "            dir_string = 'up'\n",
    "        elif self.direction == -1:\n",
    "            dir_string = 'down'\n",
    "        elif self.direction == 0 :\n",
    "            dir_string = 'No movement' \n",
    "            \n",
    "        if logs: print('Elevator moves %s from floor %d carrying %d people.' %(dir_string, self.current_floor, self.user_count))\n",
    "        \n",
    "        if strategy == 1:\n",
    "            if self.current_floor >= self.max_floors: #>= for debugging\n",
    "                self.direction = -1 #direction is either -1 or 1\n",
    "        \n",
    "            if self.current_floor == 1:\n",
    "                self.direction = +1\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 0,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "collapsed": false
   },
   "source": [
    "Definition of the building class"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 63,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
    "class Building:\n",
    "   \n",
    "    def __init__(self, elevator, users):\n",
    "#     '''\n",
    "#     Initializes the building object.\n",
    "#     Inputs:\n",
    "        \n",
    "#      * max_floors (int) The number of floors in the building.\n",
    "    \n",
    "#      * elevator (Elevator) The elevator used in the simulation.\n",
    "      \n",
    "#      * capacity (list of Passenger) A list of users used in the sim.\n",
    "       \n",
    "#      * logs (bool) Whether to print several debugging messages\n",
    "#        when running the simulation.\n",
    "#     '''\n",
    "    \n",
    "    \n",
    "        self.max_floors = elevator.max_floors #OR n_floors = int(input(\"How many floors are there in the building?\"))\n",
    "        self.elevator = elevator\n",
    "        self.users = users\n",
    "        \n",
    "        # There arrays are storages for values to be plotted later\n",
    "        self.calls_over_time = []\n",
    "        self.requests_over_time = []\n",
    "        self.count_over_time = []\n",
    "        \n",
    "    def timestep(self, elevator, users, strategy = 1, logs = False):\n",
    "#         '''\n",
    "#         This is the main control function of the simulation. It checks \n",
    "#         whether the elevator should stop or move. If it stops, it allows\n",
    "#         users to come in and out of the elevator. If it moves, it computes\n",
    "#         the appropriate path.\n",
    "#         '''\n",
    "        \n",
    "        #print(elevator.user_count, elevator.current_floor*elevator.direction, elevator.calls) #check evolution of attributes\n",
    "            \n",
    "        # Check if elevator needs to stop\n",
    "        if elevator.stop(): # If the elevator stops, do as below\n",
    "            if logs: print('Elevator stops at %d.' %(elevator.current_floor))\n",
    "            elevator.swap(users, logs = logs) # Add and drop passengers\n",
    "        else: # If elevator needs to move do as follows\n",
    "            if strategy == 2: elevator.path() # Check where it has to go\n",
    "\n",
    "            elevator.move(logs = logs, strategy = strategy) # Move a floor up or down\n",
    "\n",
    "        if random.random() < 0.1: #10% chance of adding a new call from the pool regardless of action taken\n",
    "                try:\n",
    "                    if logs: print('User %d called the elevator.' %(elevator.add_call()))\n",
    "# Nico: Why did you add .add_call() in the else?\n",
    "                    else: elevator.add_call()\n",
    "                except:\n",
    "                    pass\n",
    "    \n",
    "    def store_data(self, elevator):\n",
    "#         '''\n",
    "#         Store final attribute values.\n",
    "#         '''\n",
    "        self.calls_over_time.append(len(elevator.calls))\n",
    "        self.requests_over_time.append(len(elevator.requests))\n",
    "        self.count_over_time.append(elevator.user_count)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "collapsed": false
   },
   "source": [
    "Definition of the passenger class"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 64,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
    "\n",
    "class Passenger:\n",
    "    \n",
    "    def __init__(self, id, elevator, seed, in_elevator = False): #setting default values, otherwise specifiable\n",
    "#         '''\n",
    "#         Initializes a passenger object. A passenger has an id, an origin,\n",
    "#         a destination, a direction, and a tracker on whether they are in the elevator or not.\n",
    "#         '''\n",
    "        ifrandom.seed(id + seed) #im putting the initialization of the two attributes below so that it takes the seed change\n",
    "        \n",
    "        if random.random() < 0.3: # 30% chance the user will start from ground floor\n",
    "            self.current_floor = 1\n",
    "                destination = random.randint(1, elevator.max_floors)\n",
    "            while (destination == 1):\n",
    "            destination = random.randint(2, elevator.max_floors)\n",
    "                self.destination = destination\n",
    "\n",
    "        else: # 70% chance the user will start anywhere else\n",
    "            self.current_floor = random.randint(2, elevator.max_floors)\n",
    "    \n",
    "            if random.random() < 0.3: # 30% chance the user will want to go to ground floor\n",
    "                self.destination = 1\n",
    "            else:\n",
    "                    stination = random.randint(1, elevator.max_floors) # 70% chance the user will want to go anywhere else\n",
    "                    while (destination == self.current_floor):\n",
    "                destination = random.randint(2, elevator.max_floors)\n",
    "        self.destination = destination\n",
    "        \n",
    "\n",
    "        self.id = id\n",
    "        self.direction=np.sign(self.destination - self.current_floor) # Direction as the sign of the difference\n",
    "        self.in_elevator = in_elevator\n",
    "    \n",
    "    def __str__(self):\n",
    "        '''\n",
    "        Define how a Passenger object should be printed\n",
    "        '''\n",
    "        in_string = '' # String to represent direction ('up' or 'down')\n",
    "        if self.in_elevator == True: in_string = 'inside'\n",
    "        else: in_string = 'outside'\n",
    "        \n",
    "        dir_string = ''\n",
    "        if self.direction == 1: dir_string = 'up'\n",
    "        elif self.direction == -1: dir_string = 'down'\n",
    "        elif self.direction == 0: dir_string = 'nowhere (they are at destination)' #this shouldn't be necessary\n",
    "        \n",
    "        return('Passenger %d is %s the elevator, wanting to go %s from floor %d to floor %d.') %(self.id,in_string,dir_string,self.current_floor,self.destination)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "collapsed": false
   },
   "source": [
    "Overarching loop"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 65,
   "metadata": {
    "collapsed": false,
    "scrolled": true
   },
   "outputs": [
   ],
   "source": [
    "def simulate_elevator(passenger_count, n_floors, max_capacity, max_time, logs = False, seed = 1, strategy = 1):\n",
    "    '''\n",
    "    The main function initializes storage arrays and the main objects:\n",
    "    The Elevator, the list of Passengers, and the Building.\n",
    "    The main loop iterates over the array of time steps, storing\n",
    "    the data of the current step and calling on the .timestep()\n",
    "    method of the building object. The loop breaks if no one\n",
    "    is using the elevator.\n",
    "    Inputs:\n",
    "\n",
    "     * passenger_count (int) The numbers of users in the simulation.\n",
    "\n",
    "     * n_floors (int) The number of floors in the building.\n",
    "\n",
    "     * max_capacity (int) The maximum number of people the elevator can hold.\n",
    "\n",
    "     * max_time (int) The maximum number of time steps of the simulation.\n",
    "     \n",
    "     * logs (bool) Defaults to false. Whether to print several messages during\n",
    "       execution to help with debugging.\n",
    "       \n",
    "     * seed (int) Defaults to 1. A seed that changes the results of the\n",
    "       stochastic parts of the model.\n",
    "     \n",
    "     * strategy (int) Defaults to 1. 1 is a basic strategy where the elevator traverses\n",
    "       the entire building before changing directions. 2 is a more complex strategy that\n",
    "       optimizes the elevator's path. \n",
    "       \n",
    "     Outputs:\n",
    "     \n",
    "     * (dictionary) Includes three arrays storing the time steps of the simmulation,\n",
    "       the number of calls over time, and the number of requests over time.\n",
    "     '''\n",
    "    \n",
    "    elevator = Elevator(max_floors = n_floors, capacity = max_capacity) # Initialize elevator\n",
    "    # for line before, an alternative is: passenger_count = int(input(\"How many passengers will use the elevator?\"))\n",
    "    users = [Passenger(id = i, elevator = elevator, seed = seed) for i in range(passenger_count)] # Generate users\n",
    "    building = Building(elevator = elevator, users = users) # Initialize building\n",
    "\n",
    "    for user in users:\n",
    "        elevator.call_pool[user.id] = user.current_floor * user.direction ## IMPORTANT: This replentishes the calling queue!!\n",
    "\n",
    "    for time_step in max_time: #main loop\n",
    "        building.store_data(elevator)\n",
    "        building.timestep(elevator, users, strategy = strategy, logs = logs)\n",
    "        if len(elevator.requests)!=elevator.user_count:\n",
    "            if logs: print(len(elevator.requests),elevator.user_count)\n",
    "\n",
    "        if (time_step > 50) and (len(elevator.calls)==0) and (len(elevator.requests)==0) and (len(elevator.call_pool)==0):\n",
    "            if logs: print('Simulation over after %d timesteps.' %time_step)\n",
    "            break #end simulation if no one is using or calling the elevator\n",
    "\n",
    "    building.store_data(elevator)\n",
    "    return {'timesteps' : time_step,'calls' : building.calls_over_time, 'requests' : building.requests_over_time}"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "collapsed": false
   },
   "source": [
    "Plot of passenger statistics"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 104,
   "metadata": {
    "collapsed": false,
    "scrolled": true
   },
   "outputs": [
    {
     "data": {
      "image/png": "c460e55caaee2102fb45b86af2945e24ad2de73e"
     },
     "output_type": "execute_result"
    },
    {
     "data": {
      "image/png": "e108d09df293a6e84fcc3840cc1352791e185688"
     },
     "output_type": "execute_result"
    }
   ],
   "source": [
    "waiting = simulate_elevator(passenger_count, n_floors, max_capacity, max_time, strategy = 1)['calls']\n",
    "inside = simulate_elevator(passenger_count, n_floors, max_capacity, max_time, strategy = 1)['requests']\n",
    "\n",
    "plot_calls = plt.plot(waiting, label='Waiting for elevator')\n",
    "plt.title('Trolley elevator')\n",
    "plt.xlabel('Timestep')\n",
    "plot_requests = plt.plot(inside, label='In elevator')\n",
    "plt.xlabel('Timestep')\n",
    "plt.text(s='Average in elevator: '+str(round(np.mean(inside),2)),y=5.5,x=0)\n",
    "plt.text(s='Average waiting: '+str(round(np.mean(waiting),2)),y=5,x=0)\n",
    "plt.legend(loc=1)\n",
    "plt.show()\n",
    "\n",
    "waiting = simulate_elevator(passenger_count, n_floors, max_capacity, max_time, strategy = 2)['calls']\n",
    "inside = simulate_elevator(passenger_count, n_floors, max_capacity, max_time, strategy = 2)['requests']\n",
    "\n",
    "plot_calls = plt.plot(waiting, label='Waiting for elevator')\n",
    "plt.title('Smart elevator')\n",
    "plt.xlabel('Timestep')\n",
    "plot_requests = plt.plot(inside, label='In elevator')\n",
    "plt.xlabel('Timestep')\n",
    "plt.text(s='Average in elevator: '+str(round(np.mean(inside),2)),y=6,x=0)\n",
    "plt.text(s='Average waiting: '+str(round(np.mean(waiting),2)),y=5,x=0)\n",
    "plt.legend(loc=1)\n",
    "plt.show()\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 105,
   "metadata": {
    "collapsed": false,
    "scrolled": true
   },
   "outputs": [
    {
     "data": {
      "image/png": "cf099645ad6b843d92b2fc5594cdc867e9b2c78a"
     },
     "output_type": "execute_result"
    }
   ],
   "source": [
    "timesteps_trolley = []\n",
    "timesteps_smart = []\n",
    "\n",
    "\n",
    "for _ in range(2,100):\n",
    "    timesteps_trolley.append(simulate_elevator(passenger_count = 100, n_floors = _, max_capacity = max_capacity, max_time = max_time, strategy = 1)['timesteps'])\n",
    "    \n",
    "for _ in range(2,100):\n",
    "    timesteps_smart.append(simulate_elevator(passenger_count = 100, n_floors = _, max_capacity = max_capacity, max_time = max_time, strategy = 2)['timesteps'])\n",
    "    \n",
    "plt.plot(range(2,100),timesteps_trolley,label='Trolley elevator')\n",
    "plt.plot(range(2,100),timesteps_smart,label='Smart elevator')\n",
    "plt.xlabel('Number of floors in the building')\n",
    "plt.xticks(np.arange(0,100,5))\n",
    "plt.ylabel('Timesteps required to finish simulation')\n",
    "plt.legend(loc=1)\n",
    "plt.show()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 0,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 0,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 0,
   "metadata": {
    "collapsed": false,
    "scrolled": true
   },
   "outputs": [
    {
     "data": {
      "image/png": "26a00a8d8bab4bbaee77e4893f836787201ac5eb"
     },
     "output_type": "execute_result"
    }
   ],
   "source": [
    "timesteps_trolley = []\n",
    "timesteps_smart = []\n",
    "\n",
    "for _ in range(120):\n",
    "    timesteps_trolley.append(simulate_elevator(passenger_count = _, n_floors = n_floors, max_capacity = max_capacity, max_time = max_time, strategy = 1)['timesteps'])\n",
    "    \n",
    "for _ in range(120):\n",
    "    timesteps_smart.append(simulate_elevator(passenger_count = _, n_floors = n_floors, max_capacity = max_capacity, max_time = max_time, strategy = 2)['timesteps'])\n",
    "\n",
    "m_trolley,b_trolley = np.polyfit(range(120), timesteps_trolley, 1)\n",
    "m_smart,b_smart = np.polyfit(range(120), timesteps_smart, 1)\n",
    "\n",
    "\n",
    "    \n",
    "plt.plot(range(120),timesteps_trolley,label='Trolley elevator',linewidth=0.50)\n",
    "plt.plot([m_trolley*x + b_trolley for x in range(120)],color=u'#1f77b4',label= str(round(m_trolley,4))+'x +'+str(round(b_trolley,4)))\n",
    "plt.plot(range(120),timesteps_smart,label='Smart elevator',linewidth=0.50)\n",
    "plt.plot([m_smart*x + b_trolley for x in range(120)],color=u'#ff7f0e',label= str(round(m_smart,4))+'x +'+str(round(b_smart,4)))\n",
    "plt.plot\n",
    "plt.xlabel('Number of passengers in the building')\n",
    "plt.ylabel('Timesteps required to finish simulation')\n",
    "plt.legend(loc=1)\n",
    "plt.show()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 107,
   "metadata": {
    "collapsed": false,
    "scrolled": true
   },
   "outputs": [
    {
     "data": {
      "image/png": "6b090ac5e1f35271a2a0cd5e2d2b38e1db410700"
     },
     "output_type": "execute_result"
    }
   ],
   "source": [
    "timesteps_trolley = []\n",
    "timesteps_smart = []\n",
    "\n",
    "for _ in range(30):\n",
    "    timesteps_trolley.append(simulate_elevator(passenger_count = 100, n_floors = 20, max_capacity = _, max_time = max_time, strategy = 1)['timesteps'])\n",
    "    \n",
    "for _ in range(30):\n",
    "    timesteps_smart.append(simulate_elevator(passenger_count = 100, n_floors = 20, max_capacity = _, max_time = max_time, strategy = 2)['timesteps'])\n",
    "    \n",
    "plt.plot(range(30),timesteps_trolley,label='Trolley elevator')\n",
    "plt.plot(range(30),timesteps_smart,label='Smart elevator')\n",
    "plt.xlabel('Elevator capacity')\n",
    "plt.ylabel('Timesteps required to finish simulation')\n",
    "plt.legend(loc=1)\n",
    "plt.grid(True)\n",
    "plt.show()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 0,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 96,
   "metadata": {
    "collapsed": false,
    "scrolled": true
   },
   "outputs": [
    {
     "data": {
      "image/png": "9d15660907ef00dc633cdbb05e9e98c8e8798afd"
     },
     "output_type": "execute_result"
    }
   ],
   "source": [
    "timesteps_trolley = []\n",
    "timesteps_smart = []\n",
    "\n",
    "for _ in range(200):\n",
    "    timesteps_trolley.append(simulate_elevator(passenger_count = passenger_count, n_floors = n_floors, max_capacity = max_capacity, max_time = max_time, seed =_,strategy=1)['timesteps'])\n",
    "    \n",
    "for _ in range(200):\n",
    "    timesteps_smart.append(simulate_elevator(passenger_count = passenger_count, n_floors = n_floors, max_capacity = max_capacity, max_time = max_time, seed =_, strategy=2)['timesteps'])\n",
    "    \n",
    "plt.hist(timesteps_trolley,label='Trolley elevator',bins=9,alpha=0.5)\n",
    "plt.hist(timesteps_smart,label='Smart elevator',bins=9,alpha=0.5)\n",
    "\n",
    "plt.suptitle('Average timesteps taken: '+str(int(np.mean(timesteps_trolley)))+' (trolley) '+str(int(np.mean(timesteps_smart)))+' (smart)'+'\\nMost timesteps taken: '+str(max(timesteps_trolley))+' (trolley) '+str(max(timesteps_smart))+' (smart)')\n",
    "plt.text(s='p-value:'+str(ttest_ind(timesteps_trolley,timesteps_smart)[1]),y=50,x=1200,fontsize=12)\n",
    "plt.xlabel('Timesteps required to finish simulation')\n",
    "plt.ylabel('Relative frequency')\n",
    "plt.show()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 0,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (old Anaconda 3)",
   "env": {
    "LD_LIBRARY_PATH": "/ext/anaconda3/lib",
    "PYTHONHOME": "/ext/anaconda3/lib/python3.5",
    "PYTHONPATH": "/ext/anaconda3/lib/python3.5:/ext/anaconda3/lib/python3.5/site-packages"
   },
   "language": "python",
   "metadata": {
    "cocalc": {
     "description": "Python/R distribution for data science",
     "priority": -1,
     "url": "https://www.anaconda.com/distribution/"
    }
   },
   "name": "anaconda3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.5.5"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 0
}